__author__ = 'Dmitriy Korsakov'
__doc__ = 'RoleImage management'

import copy
import json
from scalrctl import commands
from scalrctl import click


class ReplaceRoleImage(commands.Action):

    epilog = "Example: scalr-ctl role-image replace --roleID <roleID> --imageID <ID> --newImageID <newID>"

    post_template = {
        "roleImageObject": {"image": {"id": None}, "role": {"id": None}}
    }

    def get_options(self):
        hlp = "The ID of a new image"
        newimageid = click.Option(('--newImageId', 'newimageid'), required=True, help=hlp)
        options = [newimageid, ]
        options.extend(super(ReplaceRoleImage, self).get_options())
        return options

    def pre(self, *args, **kwargs):
        """
        before request is made
        """
        newimageid = kwargs.pop("newimageid", None)
        post_data = copy.deepcopy(self.post_template)
        post_data["roleImageObject"]["image"]["id"] = newimageid
        post_data["roleImageObject"]["role"]["id"] = kwargs["roleId"]
        kv = {"import-data": post_data}
        kv.update(kwargs)
        arguments, kw = super(ReplaceRoleImage, self).pre(*args, **kv)
        return arguments, kw

    def post(self, response):
        """
        after request is made
        """
        try:
            obj = json.loads(response)
            if "errors" not in obj:
                roleid = obj["data"]["role"]["id"]
                imageid = obj["data"]["image"]["id"]
                click.echo("Role %s now contains new image %s." % (roleid, imageid))
        except:
            pass
        return response


class CreateRoleImage(commands.Action):

    epilog = "Example: scalr-ctl role-image create --roleID <roleID> --imageID <ID>"

    post_template = {
        "roleImageObject": {"image": {"id": None}, "role": {"id": None}}
    }

    def get_options(self):
        hlp = "The ID of a new image"
        imageid = click.Option(('--imageId', 'imageId'), required=True, help=hlp)
        options = [imageid, ]
        options.extend(super(CreateRoleImage, self).get_options())
        return options

    def pre(self, *args, **kwargs):
        """
        before request is made
        """
        imageid = kwargs.get("imageId")
        post_data = copy.deepcopy(self.post_template)
        post_data["roleImageObject"]["image"]["id"] = imageid
        post_data["roleImageObject"]["role"]["id"] = kwargs["roleId"]
        kv = {"import-data": post_data}
        kv.update(kwargs)
        arguments, kw = super(CreateRoleImage, self).pre(*args, **kv)
        return arguments, kw



__author__ = 'Dmitriy Korsakov'
__doc__ = 'Image management'

import copy
import json
from scalrctl import commands
from scalrctl import click


class ChangeImageAttrs(commands.Action):
    mutable_body_parts = ["name"]
    prompt_for = ["imageId"]


class RegisterImage(commands.Action):

    def pre(self, *args, **kwargs):
        #XXX: this code does not work yet!
        if 'imageId' not in kwargs:
            kwargs["image"] = click.termui.prompt("Image object JSON")

        return super(RegisterImage, self).pre(*args, **kwargs)


class RetrieveImage(commands.Action):
    prompt_for = ["imageId"]


class ReplaceImage(commands.Action):

    epilog = "Example: scalr-ctl image replace --imageID <ID> --newImageID <newID> --deprecateOldImage --scope <SCOPE>"

    post_template = {
        "replaceImageRequest": {"scope": None, "newImage": {"id": None}},
    }

    _default_scope = "account"

    def get_options(self):
        hlp = "The ID of a new image"
        newimageid = click.Option(('--newImageId', 'newimageid'), required=True, help=hlp)
        deprecation_help = "If the value is true Scalr will mark source Image as deprecated."
        deprecate = click.Option(('--deprecateOldImage', 'deprecate'), help=deprecation_help, is_flag=True)
        scope_help = "Make a replacement for all Roles from the selected scopes. "
        scope_help += "If you choose to make a replacement including lower scope higher scope values will be chosen too"
        scope = click.Option(('--scope', 'scope'), default=self._default_scope, help=scope_help)

        options = [newimageid, deprecate, scope]
        options.extend(super(ReplaceImage, self).get_options())
        for option in options:
            #if option.name == "interactive":
            if option.name == "stdin":
                options.remove(option)
        return options

    def pre(self, *args, **kwargs):
        """
        before request is made
        """
        newimageid = kwargs.pop("newimageid", None)
        post_data = copy.deepcopy(self.post_template)
        post_data["replaceImageRequest"]["newImage"]["id"] = newimageid
        scope = kwargs.pop("scope")
        post_data["replaceImageRequest"]["scope"] = scope,
        kv = {"import-data": post_data}
        kv.update(kwargs)
        arguments, kw = super(ReplaceImage, self).pre(*args, **kv)
        return arguments, kw

    def post(self, response):
        """
        after request is made
        """

        try:
            obj = json.loads(response)
            if "errors" not in obj:
                #roleid = obj["data"]["role"]["id"]
                imageid = obj["data"]["newImage"]["id"]
                click.echo("New image set: %s." % imageid)
        except:
            pass
        return response


class ReplaceImageGlobal(ReplaceImage):

    _default_scope = "scalr"


"""
Export Scalr objects.
"""
import os
import copy
import datetime
import json
import pydoc

import yaml

from scalrctl import click, commands, defaults, settings


__author__ = 'Dmitriy Korsakov, Sergey Babak'


class Export(commands.Action):

    relations = {}

    def _get_custom_options(self):
        # Disable output modifiers
        options = []
        debug = click.Option(('--debug', 'debug'), is_flag=True,
                             default=False, help="Print debug messages")
        options.append(debug)
        return options

    @staticmethod
    def _get_param(parent, child, key):
        head, _, tail = key.partition('.')
        if head == 'child':
            return reduce(dict.__getitem__, tail.split('.'), child)
        elif head == 'parent':
            return reduce(dict.__getitem__, tail.split('.'), parent)
        else:
            raise click.ClickException("Invalid key: \"{}\"".format(key))

    def _get_relations(self, parent):

        with open(os.path.join(os.path.dirname(__file__),
                               '../scheme/scheme.json')) as fp:
            scheme = json.load(fp)

        data = []
        for relation, relation_values in self.relations.items():
            list_data = scheme[relation]['list']
            list_action = commands.Action(
                name=relation,
                route=list_data['route'],
                http_method=list_data['http-method'],
                api_level=list_data['api_level'],
            )

            get_data = scheme[relation]['get']
            # TODO: recursive export
            # export_cls = pydoc.locate(
            #   scheme['export'][relation]['class']
            # ) if 'class' in scheme['export'][relation] else Export
            get_action = Export(
                name=relation,
                route=get_data['route'],
                http_method=get_data['http-method'],
                api_level=get_data['api_level'],
            )

            list_kwargs = {'hide_output': True}
            for key, value in relation_values['list'].items():
                list_kwargs[key] = self._get_param(parent, None, value)

            list_action_resp = list_action.run(**list_kwargs)
            resp_json = json.loads(list_action_resp)

            for obj_data in resp_json['data']:
                get_kwargs = {'hide_output': True}
                for key, value in relation_values['get'].items():
                    get_kwargs[key] = self._get_param(parent, obj_data, value)
                resp = get_action.run(**get_kwargs)
                data.extend(resp)

        return data

    def run(self, *args, **kwargs):
        hide_output = kwargs.pop('hide_output', False)

        kv = kwargs.copy()
        kv['hide_output'] = True
        response = super(Export, self).run(*args, **kv)

        kwargs['envId'] = settings.envId
        kv = copy.deepcopy(kwargs)
        for item in ('debug', 'nocolor', 'transformation'):
            if item in kv:
                del kv[item]
        uri = self._request_template.format(**kwargs)

        scalrctl_meta = {
            'API_VERSION': settings.API_VERSION,
            'envId': settings.envId,
            'DATE': datetime.datetime.utcnow().strftime('%Y-%m-%d %H:%M:%S'),
            'API_HOST': settings.API_HOST,
            'API_LEVEL': self.api_level,
            'METHOD': self.http_method,
            'ROUTE': self.route,
            'URI': uri,
            'ACTION': self.name,
            'ARGUMENTS': (args, kv),
            'SCALRCTL_VERSION': defaults.VERSION,
        }

        try:
            response_json = json.loads(response)
        except ValueError as e:
            if settings.debug_mode:
                raise
            raise click.ClickException(str(e))

        response_json['meta']['scalrctl'] = scalrctl_meta

        result = [response_json, ]
        if self.relations:
            relations = self._get_relations(response_json['data'])
            result.extend(relations)

        def _order(arg):
            action_name = arg['meta']['scalrctl']['ACTION']
            return self.relations.get(action_name, {}).get('order', 0)

        result = sorted(result, key=_order)

        if not hide_output:
            dump = yaml.safe_dump(
                result,
                encoding='utf-8',
                allow_unicode=True,
                default_flow_style=False
            )
            click.echo(dump)

        return result


class ExportFarmRoleGlobalVariable(Export):
    prompt_for = ['roleId', 'globalVariableName']


class ExportImage(Export):
    prompt_for = ['imageId']


class ExportScript(Export):

    relations = {
        'script-versions': {
            'get': {
                'scriptId': 'parent.id',
                'scriptVersionNumber': 'child.version',
            },
            'list': {
                'scriptId': 'parent.id'
            }
        },
    }


class ExportRole(Export):

    relations = {
        'role-categories': {
            'order': -1,
            'get': {
                'roleCategoryId': 'child.id',
            },
            'list': {
                'scope': 'parent.scope',
                'id': 'parent.category.id',
            },
        },
        'role-orchestration-rules': {
            'get': {
                'roleId': 'parent.id',
                'orchestrationRuleId': 'child.id',
            },
            'list': {
                'roleId': 'parent.id'
            }
        },
        'role-global-variables': {
            'get': {
                'globalVariableName': 'child.name',
                'roleId': 'parent.id'
            },
            'list': {
                'roleId': 'parent.id',
                'declaredIn': 'parent.scope'
            }
        },
        'role-images': {
            'get': {
                'roleId': 'child.role.id',
                'imageId': 'child.image.id',
            },
            'list': {
                'roleId': 'parent.id'
            },
        },
    }


class ExportRoleImage(Export):

    relations = {
        'images': {
            'order': -1,
            'get': {
                'imageId': 'parent.id',
            },
            'list': {
                'id': 'parent.id',
            },
        },
    }



# -*- coding: utf-8 -*-
import os
import sys
import time
import json
import itertools
import threading
import traceback

import six
import yaml
import requests

from scalrctl import click, defaults, settings, commands

__author__ = 'Dmitriy Korsakov, Sergey Babak'


class _spinner(object):

    @staticmethod
    def draw(event):
        cursor = itertools.cycle('|/-\\')
        while not event.isSet():
            sys.stdout.write(next(cursor))
            sys.stdout.flush()
            time.sleep(0.1)
            sys.stdout.write('\b')
        sys.stdout.write(' ')
        sys.stdout.flush()

    def __init__(self):
        self.event = threading.Event()
        self.thread = threading.Thread(target=_spinner.draw,
                                       args=(self.event,))
        self.thread.daemon = True

    def __enter__(self):
        self.thread.start()

    def __exit__(self, type, value, traceback):
        self.event.set()
        self.thread.join()


def _get_spec_path(api_level, extension):
    return os.path.join(defaults.CONFIG_DIRECTORY,
                        '{}.{}'.format(api_level, extension))


def _is_spec_exists(api_level, extension):
    return os.path.exists(_get_spec_path(api_level, extension))


def _load_yaml_spec(api_level):
    spec_url = "{0}://{1}/api/{2}.{3}.yml".format(settings.API_SCHEME,
                                                  settings.API_HOST,
                                                  api_level,
                                                  settings.API_VERSION)
    resp = requests.get(spec_url, verify=settings.SSL_VERIFY_PEER)
    return resp.text if resp.status_code == 200 else None


def _read_spec(spec_path):
    text = None
    if os.path.exists(spec_path):
        with open(spec_path, "r") as fp:
            text = fp.read()
    return text


def _write_spec(spec_path, text):
    with open(spec_path, "w") as fp:
        fp.write(text)


def _update_spec(api_level):
    """
    Downloads yaml spec and converts it to JSON
    Both files are stored in configuration directory.
    """

    try:
        yaml_spec_text = _load_yaml_spec(api_level)
        if not yaml_spec_text:
            raise Exception('Can\'t load spec file')

        try:
            struct = yaml.load(yaml_spec_text)
            json_spec_text = json.dumps(struct)
        except (KeyError, TypeError, yaml.YAMLError) as e:
            six.reraise(type(e), "Swagger specification is not valid:\n{}"
                        .format(traceback.format_exc()))

        yaml_spec_path = _get_spec_path(api_level, 'yaml')
        json_spec_path = _get_spec_path(api_level, 'json')

        old_yaml_spec_text = _read_spec(yaml_spec_path)

        # update yaml spec
        if yaml_spec_text != old_yaml_spec_text:
            _write_spec(yaml_spec_path, yaml_spec_text)

        # update json spec and routes
        _write_spec(json_spec_path, json_spec_text)

        return True, None
    except Exception as e:
        return False, e.message or 'Unknown reason'


class UpdateScalrCTL(commands.BaseAction):

    def run(self, *args, **kwargs):
        update()

    def get_description(self):
        return "Fetch new API specification if available."


def is_update_required():
    """
    Determine if spec update is needed.
    """

    # prevent from running 'update' more than once
    if 'update' in sys.argv:
        return False
    else:
        exists = [_is_spec_exists(api, 'yaml') and
                  _is_spec_exists(api, 'json') for api in defaults.API_LEVELS]
        return not all(exists)


def update():
    """
    Update spec for all available APIs.
    """

    amount = len(defaults.API_LEVELS)

    for index, api_level in enumerate(defaults.API_LEVELS, 1):

        click.echo('[{}/{}] Updating specifications for {} API ... '
                   .format(index, amount, api_level), nl=False)

        with _spinner():
            success, fail_reason = _update_spec(api_level)

        if success:
            click.secho('Done', fg='green')
        else:
            click.secho('Failed: {}'.format(fail_reason), fg='red')




__author__ = 'Dmitriy Korsakov'

import os
import site
import shutil

from scalrctl import click
from scalrctl import defaults


PROGNAME = "scalr-ctl"
AUTOCOMPLETE_FNAME = "path.bash.inc"
AUTOCOMPLETE_CONTENT = "_%s_COMPLETE=source %s" % (PROGNAME.upper().replace("-", "_"), PROGNAME)
AUTOCOMPLETE_PATH = os.path.join(os.path.expanduser(defaults.CONFIG_DIRECTORY), AUTOCOMPLETE_FNAME)


def setup_bash_complete():
    if "nt" == os.name: # Click currently only supports completion for Bash.
        return

    bashrc_path = os.path.expanduser("~/.bashrc")
    bashprofile_path = os.path.expanduser("~/.bash_profile")
    startup_path = bashprofile_path if os.path.exists(bashprofile_path) else bashrc_path
    startup_path = click.prompt("Enter path to an rc file to update, or leave blank to use", default=startup_path, err=True)
    if not os.path.exists(startup_path):
        click.echo("%s not found." % startup_path)
        return

    startupfile_content = open(startup_path, "r").read()

    if AUTOCOMPLETE_PATH not in startupfile_content:
        confirmed = click.confirm("Modify profile to update your $PATH and enable bash completion?", default=True, err=True)

        if confirmed:
            with open(AUTOCOMPLETE_PATH, "w") as fp:
                fp.write(AUTOCOMPLETE_CONTENT)

            backup_path = startup_path + ".backup"
            click.echo("Backing up [%s] to [%s]." % (startup_path, backup_path))
            shutil.copy(startup_path, backup_path)

            comment = "# The next lines enable bash completion for %s." % PROGNAME
            local_bindir = os.path.expanduser(os.path.join(site.USER_BASE, "bin"))
            local_binpath = os.path.join(local_bindir, PROGNAME)
            source_line = 'eval "$(%s)"' % AUTOCOMPLETE_CONTENT

            if os.path.exists(local_binpath):
                msg = "Do you want to add %s to your $PATH?" % local_bindir
                export_confirmed = click.confirm(msg, default=True, err=True)
                if export_confirmed:
                    export_line = "export PATH=$PATH:%s" % local_bindir  # [ST-112]
                else:
                    source_line = ''
            else:
                export_line = ''

            scalr_configs = filter(None, (comment, export_line, source_line))

            # removes old scalr configs
            is_not_scalr_line = lambda line: not any(sc == line for sc in scalr_configs)
            startupfile_content = startupfile_content.rstrip('\n').split('\n')
            startupfile_content = filter(is_not_scalr_line, startupfile_content)

            if len(scalr_configs) > 1:
                with open(startup_path, "w") as fp:
                    fp.write('\n'.join(startupfile_content).rstrip('\n') + '\n\n')
                    fp.write('\n'.join(scalr_configs) + '\n')

            click.echo("Start a new shell for the changes to take effect.")



__author__ = 'Dmitriy Korsakov'
__doc__ = 'Manage FarmRoles'

from scalrctl import commands


class LaunchServer(commands.Action):

    post_template = {}

    def pre(self, *args, **kwargs):
        """
        before request is made
        """
        kv = {"import-data": {}}
        kv.update(kwargs)
        arguments, kw = super(LaunchServer, self).pre(*args, **kv)
        return arguments, kw


# -*- coding: utf-8 -*-
import binascii
import hashlib
import hmac
import json
import time

import requests
import yaml
from six.moves.urllib.parse import quote, urlunsplit

from scalrctl import click, settings
from scalrctl.compat import urlencode

__author__ = 'Dmitriy Korsakov, Sergey Babak'


"""
import logging

# these two lines enable debugging at httplib level
# (requests->urllib3->httplib) you will see the REQUEST, including
# HEADERS and DATA, and RESPONSE with HEADERS but without DATA.
# the only thing missing will be the response.body which is not logged.

import httplib
httplib.HTTPConnection.debuglevel = 1

logging.basicConfig() # you need to initialize logging,
                      # otherwise you will not see anything from requests
logging.getLogger().setLevel(logging.DEBUG)
requests_log = logging.getLogger("requests.packages.urllib3")
requests_log.setLevel(logging.DEBUG)
requests_log.propagate = True
"""


try:
    requests.packages.urllib3.disable_warnings()
except:
    pass


def _key_pair(api_level='user'):
    """
    Returns key pair(key id and secret key) for specified API scope.
    """

    if api_level in ('global', ):
        api_key_id = settings.GLOBAL_SCOPE_API_KEY_ID
        secret_key = settings.GLOBAL_SCOPE_API_SECRET_KEY
        no_key_msg = "API {} for global scope is not configured. " \
            "Please run 'scalr-ctl configure --with-global-scope' to change default " \
            "authentication settings."
        assert api_key_id, no_key_msg.format("Key ID")
        assert secret_key, no_key_msg.format("Secret Key")
    elif api_level in ('user', 'account'):
        api_key_id = settings.API_KEY_ID
        secret_key = settings.API_SECRET_KEY
        no_key_msg = "API {} is not configured. Please specify option " \
            "{} or run 'scalr-ctl configure' to change default " \
            "authentication settings."
        assert api_key_id, no_key_msg.format("Key ID", "--key_id")
        assert secret_key, no_key_msg.format("Secret Key", "--secret_key")
    else:
        raise click.ClickException('Invalid Scalr API level')

    return api_key_id, secret_key


def request(method, api_level, request_uri, payload=None, data=None):
    """
    Makes request to Scalr API.
    """

    time_iso8601 = time.strftime('%Y-%m-%dT%H:%M:%S.000Z', time.gmtime())

    try:
        api_key_id, secret_key = _key_pair(api_level=api_level)

        query_string = urlencode(
            sorted(payload.items()),
            quote_via=quote
        ) if payload else ''

        body = json.dumps(yaml.safe_load(data)) if data else ''  # XXX

        string_to_sign = '\n'.join((
            method.upper(),
            time_iso8601,
            request_uri,
            query_string,
            body
        ))

        digest = hmac.new(
            secret_key.encode('UTF-8'),
            string_to_sign.encode('UTF-8'),
            hashlib.sha256
        ).digest()

        signature = '{} {}'.format(
            settings.SIGNATURE_VERSION,
            binascii.b2a_base64(digest).strip().decode('UTF-8')
        )

        headers = dict()
        headers['Content-Type'] = 'application/json; charset=utf-8'
        headers['X-Scalr-Key-Id'] = api_key_id
        headers['X-Scalr-Date'] = time_iso8601
        headers['X-Scalr-Signature'] = signature
        # if hasattr(settings, "API_DEBUG") and settings.API_DEBUG:
        #    headers['X-Scalr-Debug'] = 1

        url = urlunsplit((
            settings.API_SCHEME,
            settings.API_HOST,
            request_uri,
            '',
            ''
        ))

        if settings.debug_mode:
            click.echo('API HOST: {}\n'
                       'stringToSign: {}\n'
                       'Headers: {}\n'.format(
                           settings.API_HOST,
                           string_to_sign,
                           json.dumps(headers, indent=2))
                       )

        resp = requests.request(
            method.lower(),
            url,
            data=body,
            params=payload,
            headers=headers,
            verify=settings.SSL_VERIFY_PEER
        )
        result = resp.text

    except (Exception, BaseException) as e:
        if settings.debug_mode:
            raise
        raise click.ClickException(str(e))
    return result



# -*- coding: utf-8 -*-
import os
import json
import re

from scalrctl import click, defaults

__author__ = 'Sergey Babak'


DOCS_HOST = 'https://api-explorer.scalr.com'

EXCLUDES = [
    '/{envId}/farm-roles/{farmRoleId}/servers/',
    '/{envId}/servers/',
]

DEFAULTS = {
    'string': '',
    'boolean': True,
    'integer': 1,
    'number': 1,
    'array': []
}


def _read_spec(api_level, extension="json"):
    text = None
    spec_path = os.path.join(defaults.CONFIG_DIRECTORY,
                             '{}.{}'.format(api_level, extension))
    if os.path.exists(spec_path):
        with open(spec_path, "r") as fp:
            text = fp.read()
    return text


def _item_by_ref(spec_data, ref):
    _, _, definition = ref.strip('/').split('/')
    return spec_data['definitions'][definition]


def _generate_params(spec_data, schema):
    params = {}

    if '$ref' in schema:
        schema = _item_by_ref(spec_data, schema['$ref'])

    if 'properties' in schema:
        for p_key, p_value in schema['properties'].items():
            if p_value.get('readOnly'):
                continue
            if '$ref' in p_value:
                sub_item = _item_by_ref(spec_data, p_value['$ref'])
                params[p_key] = _generate_params(spec_data, sub_item)
            else:
                if 'enum' in p_value:
                    params[p_key] = p_value['enum'][0]
                else:
                    params[p_key] = DEFAULTS[p_value['type']]

    return params


def generate_post_data(spec_data, endpoint):
    """
    Generates POST data for specified API endpoint.
    """

    if endpoint in spec_data['paths']:
        params_spec = spec_data['paths'].get(endpoint)
    else:
        raise click.ClickException('API endpoint {} does not found'.format(endpoint))

    if 'post' in params_spec:
        if 'parameters' in params_spec['post']:
            schema = params_spec['post']['parameters'][0]['schema']
            post_data = _generate_params(spec_data, schema)
        else:
            post_data = {}
    else:
        raise click.ClickException('POST method for endpoint {} does not exist'
                        .format(endpoint))

    return post_data


def get_definition(spec_data, endpoint):
    """
    Returns object name by endpoint.
    """

    if endpoint in spec_data['paths']:
        endpoint_spec = spec_data['paths'].get(endpoint)
    else:
        raise click.ClickException('API endpoint {} does not found'.format(endpoint))

    for param in endpoint_spec['post'].get('parameters', ''):
        if '$ref' in param.get('schema', ''):
            return param.get('schema')['$ref'].split('/')[-1]


def get_doc_url(api_level, endpoint):
    """
    Returns URL to documentation by API endpoint.
    """

    endpoint = endpoint.strip('/')

    # finds all path parameters
    params = re.findall(r'[{\[].*?}', endpoint)

    # creates postfix from last parameter
    last_param = params[-1] if params and endpoint.endswith(params[-1]) else ''
    postfix = '_{}'.format(last_param[1:-1]) if last_param else ''

    # removes all parameters from documentation path
    path = '/'.join(filter(lambda l: l not in params, endpoint.split('/')))

    return '{}/{}/{}/post{}.html'.format(DOCS_HOST, api_level, path, postfix)


def create_post_example(api_level, endpoint):
    """
    Returns example for POST request.
    """
    if endpoint in EXCLUDES:
        raise click.ClickException('Invalid API endpoint')

    spec_data = json.loads(_read_spec(api_level))

    post_data = generate_post_data(spec_data, endpoint)
    object_name = get_definition(spec_data, endpoint)
    doc_url = get_doc_url(api_level, endpoint)

    example = ("The body must be a valid {name} object. "
               "Example value:\n{post_data}\n"
               "See more at {doc_url}\n\n"
               "Type your {name} object below this line. "
               "The above text will not be sent to the API server.").format(
        name=object_name,
        post_data=json.dumps(post_data, indent=2),
        doc_url=doc_url,
    )
    example = '\n'.join(['# {}'.format(line) for line in example.split('\n')])
    return example



__author__ = 'Dmitriy Korsakov'
__doc__ = 'Server management'
import copy
import json
import time

from scalrctl import commands
from scalrctl import click


class ExecuteScript(commands.Action):

    epilog = "Example: scalr-ctl script execute --serverId <ID> --scriptId <ID> --blocking --timeout 30"

    post_template = {"scriptExecutionRequestObject": {"server": None, "blocking": False, "timeout": None}}

    def get_options(self):
        blocking_hlp = "If it is set Scalr Agent will wait for your Script to finish \
        executing before firing and processing further events."
        blocking = click.Option(('--blocking', 'blocking'), is_flag=True, default=False, help=blocking_hlp)

        server_id_hlp = "Identifier of the Server"
        server_id = click.Option(('--serverId', 'serverId'), required=True, help=server_id_hlp)

        timeout_hlp = "How many secconds should Scalr Agent wait before aborting the execution of this Script."
        timeout = click.Option(('--timeout', 'timeout'), required=False, help=timeout_hlp)

        nowait_hlp = "Do not wait for script execution to finish"
        nowait = click.Option(('--nowait', 'nowait'), required=False, help=nowait_hlp)

        options = [blocking, server_id, timeout, nowait]
        options.extend(super(ExecuteScript, self).get_options())
        return options


    def pre(self, *args, **kwargs):
        """
        before request is made
        """
        blocking = kwargs.pop("blocking", False)
        server_id = kwargs.pop("serverId")
        timeout = kwargs.pop("timeout", False)
        post_data = copy.deepcopy(self.post_template)
        post_data["scriptExecutionRequestObject"]["blocking"] = blocking
        post_data["scriptExecutionRequestObject"]["server"] = server_id
        post_data["scriptExecutionRequestObject"]["timeout"] = timeout
        kv = {"import-data": post_data, "nowait": kwargs.pop("nowait", False)}
        kv.update(kwargs)
        arguments, kw = super(ExecuteScript, self).pre(*args, **kv)
        return arguments, kw

    def run(self, *args, **kwargs):
        nowait = kwargs.pop("nowait", False)
        result = super(ExecuteScript, self).run(*args, **kwargs)
        if not nowait:
            result_json = json.loads(result)
            execution_status_id = result_json["data"]["id"]
            cls = commands.Action
            action = cls(name=self.name,
                         route="/{envId}/script-executions/{scriptExecutionId}/",
                         http_method="get",
                         api_level=self.api_level)
            status = "running"
            click.echo("Checking script execution status..")
            while status == "running":
                data = action.run(**{"scriptExecutionId": execution_status_id})
                data_json = json.loads(data)
                status = data_json["data"]["status"]
                time.sleep(1)

        return result


class ExecuteScriptVersion(ExecuteScript):

    epilog = "Example: scalr-ctl script-version execute \
    --serverId <ID> --scriptId <ID> --scriptVersionNumber <num>--blocking --timeout 30"
